@@ -2,6 +2,7 @@ package assisted_installer_controller
 
 import (
 	"context"
+	"encoding/base64"
 	"encoding/json"
 	"fmt"
 	"io"
@@ -15,7 +16,8 @@ import (
 	"sync/atomic"
 	"time"
 
-	metal3v1alpha1 "github.com/metal3-io/baremetal-operator/pkg/apis/metal3/v1alpha1"
+	"github.com/hashicorp/go-version"
+	metal3v1alpha1 "github.com/metal3-io/baremetal-operator/apis/metal3.io/v1alpha1"
 	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 	certificatesv1 "k8s.io/api/certificates/v1"
@@ -24,6 +26,7 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
 	"github.com/openshift/assisted-installer/src/common"
+	"github.com/openshift/assisted-installer/src/config"
 	"github.com/openshift/assisted-installer/src/inventory_client"
 	"github.com/openshift/assisted-installer/src/k8s_client"
 	"github.com/openshift/assisted-installer/src/ops"
@@ -35,20 +38,22 @@ import (
 const (
 	// We retry 10 times in 30sec interval meaning that we tolerate the operator to be in failed
 	// state for 5minutes.
-	failedOperatorRetry       = 10
-	generalWaitTimeoutInt     = 30
-	controllerLogsSecondsAgo  = 120 * 60
+	failedOperatorRetry   = 10
+	generalWaitTimeoutInt = 30
+	// zero means -> bring all logs
+	controllerLogsSecondsAgo  = 0
 	consoleOperatorName       = "console"
 	ingressConfigMapName      = "default-ingress-cert"
 	ingressConfigMapNamespace = "openshift-config-managed"
 	dnsServiceName            = "dns-default"
 	dnsServiceNamespace       = "openshift-dns"
 	dnsOperatorNamespace      = "openshift-dns-operator"
+	maxFetchAttempts          = 5
 	maxDeletionAttempts       = 5
 	maxDNSServiceIPAttempts   = 45
 	KeepWaiting               = false
 	ExitWaiting               = true
-	customManifestsFile       = "custom_manifests.yaml"
+	customManifestsFile       = "custom_manifests.json"
 	kubeconfigFileName        = "kubeconfig-noingress"
 )
 
@@ -61,6 +66,7 @@ var (
 	CompleteTimeout          = 30 * time.Minute
 	DNSAddressRetryInterval  = 20 * time.Second
 	DeletionRetryInterval    = 10 * time.Second
+	FetchRetryInterval       = 10 * time.Second
 	LongWaitTimeout          = 10 * time.Hour
 	CVOMaxTimeout            = 3 * time.Hour
 )
@@ -70,16 +76,21 @@ var (
 // as a first step it will wait till nodes are added to cluster and update their status to Done
 
 type ControllerConfig struct {
-	ClusterID             string `envconfig:"CLUSTER_ID" required:"true" `
-	URL                   string `envconfig:"INVENTORY_URL" required:"true"`
-	PullSecretToken       string `envconfig:"PULL_SECRET_TOKEN" required:"true"`
-	SkipCertVerification  bool   `envconfig:"SKIP_CERT_VERIFICATION" required:"false" default:"false"`
-	CACertPath            string `envconfig:"CA_CERT_PATH" required:"false" default:""`
-	Namespace             string `envconfig:"NAMESPACE" required:"false" default:"assisted-installer"`
-	OpenshiftVersion      string `envconfig:"OPENSHIFT_VERSION" required:"true"`
-	HighAvailabilityMode  string `envconfig:"HIGH_AVAILABILITY_MODE" required:"false" default:"Full"`
-	WaitForClusterVersion bool   `envconfig:"CHECK_CLUSTER_VERSION" required:"false" default:"false"`
-	MustGatherImage       string `envconfig:"MUST_GATHER_IMAGE" required:"false" default:""`
+	ClusterID               string `envconfig:"CLUSTER_ID" required:"true"`
+	URL                     string `envconfig:"INVENTORY_URL" required:"true"`
+	PullSecretToken         string `envconfig:"PULL_SECRET_TOKEN" required:"true" secret:"true"`
+	SkipCertVerification    bool   `envconfig:"SKIP_CERT_VERIFICATION" required:"false" default:"false"`
+	CACertPath              string `envconfig:"CA_CERT_PATH" required:"false" default:""`
+	Namespace               string `envconfig:"NAMESPACE" required:"false" default:"assisted-installer"`
+	OpenshiftVersion        string `envconfig:"OPENSHIFT_VERSION" required:"true"`
+	HighAvailabilityMode    string `envconfig:"HIGH_AVAILABILITY_MODE" required:"false" default:"Full"`
+	WaitForClusterVersion   bool   `envconfig:"CHECK_CLUSTER_VERSION" required:"false" default:"false"`
+	MustGatherImage         string `envconfig:"MUST_GATHER_IMAGE" required:"false" default:""`
+	DryRunEnabled           bool   `envconfig:"DRY_ENABLE" required:"false" default:"false"`
+	DryFakeRebootMarkerPath string `envconfig:"DRY_FAKE_REBOOT_MARKER_PATH" required:"false" default:""`
+	DryRunClusterHostsPath  string `envconfig:"DRY_CLUSTER_HOSTS_PATH"`
+	// DryRunClusterHostsPath gets read parsed into ParsedClusterHosts by DryParseClusterHosts
+	ParsedClusterHosts config.DryClusterHosts
 }
 type Controller interface {
 	WaitAndUpdateNodesStatus(status *ControllerStatus)
@@ -100,6 +111,14 @@ type controller struct {
 	kc     k8s_client.K8SClient
 }
 
+// manifest store the operator manifest used by assisted-installer to create CRs of the OLM:
+type manifest struct {
+	// name of the operator the CR manifest we want create
+	Name string
+	// content of the manifest of the opreator
+	Content string
+}
+
 func NewController(log *logrus.Logger, cfg ControllerConfig, ops ops.Ops, ic inventory_client.InventoryClient, kc k8s_client.K8SClient) *controller {
 	return &controller{
 		log:              log,
@@ -183,6 +202,7 @@ func (c *controller) waitAndUpdateNodesStatus() bool {
 	log := utils.RequestIDLogger(ctxReq, c.log)
 
 	assistedNodesMap, err := c.ic.GetHosts(ctxReq, log, ignoreStatuses)
+	knownIpAddresses := common.BuildHostsMapIPAddressBased(assistedNodesMap)
 	if err != nil {
 		log.WithError(err).Error("Failed to get node map from the assisted service")
 		return KeepWaiting
@@ -212,36 +232,45 @@ func (c *controller) waitAndUpdateNodesStatus() bool {
 		return KeepWaiting
 	}
 	for _, node := range nodes.Items {
-		host, ok := hostsInProgressMap[strings.ToLower(node.Name)]
+		host, ok := common.HostMatchByNameOrIPAddress(node, hostsInProgressMap, knownIpAddresses)
 		if !ok {
-			if _, ok := assistedNodesMap[strings.ToLower(node.Name)]; !ok {
-				log.Warnf("Node %s is not in inventory hosts", strings.ToLower(node.Name))
-			}
-
+			log.Warnf("Node %s is not in inventory hosts", strings.ToLower(node.Name))
 			continue
 		}
-		if common.IsK8sNodeIsReady(node) {
-			log.Infof("Found new ready node %s with inventory id %s, kubernetes id %s, updating its status to %s",
-				node.Name, host.Host.ID.String(), node.Status.NodeInfo.SystemUUID, models.HostStageDone)
-			if err := c.ic.UpdateHostInstallProgress(ctxReq, host.Host.ID.String(), models.HostStageDone, ""); err != nil {
+
+		if host.Host.Progress.CurrentStage == models.HostStageConfiguring {
+			log.Infof("Found new joined node %s with inventory id %s, kubernetes id %s, updating its status to %s",
+				node.Name, host.Host.ID.String(), node.Status.NodeInfo.SystemUUID, models.HostStageJoined)
+			if err := c.ic.UpdateHostInstallProgress(ctxReq, host.Host.InfraEnvID.String(), host.Host.ID.String(), models.HostStageJoined, ""); err != nil {
 				log.WithError(err).Errorf("Failed to update node %s installation status", node.Name)
 				continue
 			}
-		} else if host.Host.Progress.CurrentStage == models.HostStageConfiguring {
-			log.Infof("Found new joined node %s with inventory id %s, kubernetes id %s, updating its status to %s",
-				node.Name, host.Host.ID.String(), node.Status.NodeInfo.SystemUUID, models.HostStageJoined)
-			if err := c.ic.UpdateHostInstallProgress(ctxReq, host.Host.ID.String(), models.HostStageJoined, ""); err != nil {
+		}
+
+		if common.IsK8sNodeIsReady(node) {
+			log.Infof("Found new ready node %s with inventory id %s, kubernetes id %s, updating its status to %s",
+				node.Name, host.Host.ID.String(), node.Status.NodeInfo.SystemUUID, models.HostStageDone)
+			if err := c.ic.UpdateHostInstallProgress(ctxReq, host.Host.InfraEnvID.String(), host.Host.ID.String(), models.HostStageDone, ""); err != nil {
 				log.WithError(err).Errorf("Failed to update node %s installation status", node.Name)
 				continue
 			}
 		}
 	}
-	c.updateConfiguringStatusIfNeeded(assistedNodesMap)
+
+	// Since the host statuses may have changed due to the above loop,
+	// we need to get the updated list of hosts again so we don't operate
+	// on stale data.
+	assistedNodesMapUpdated, err2 := c.ic.GetHosts(ctxReq, log, ignoreStatuses)
+	if err2 != nil {
+		log.WithError(err2).Error("Failed to get node map from the assisted service")
+		return KeepWaiting
+	}
+
+	c.updateConfiguringStatusIfNeeded(assistedNodesMapUpdated)
 	return KeepWaiting
 }
 
 func (c *controller) HackDNSAddressConflict(wg *sync.WaitGroup) {
-
 	c.log.Infof("Making sure service %s can reserve the .10 address", dnsServiceName)
 
 	defer func() {
@@ -254,13 +283,14 @@ func (c *controller) HackDNSAddressConflict(wg *sync.WaitGroup) {
 		return
 	}
 
-	ip, _, _ := net.ParseCIDR(networks[0])
-	ip4 := ip.To4()
-	if ip4 == nil {
-		c.log.Infof("Service network is IPv6: %s, skipping the .10 address hack", ip)
+	netIp, _, _ := net.ParseCIDR(networks[0])
+	ip := netIp.To16()
+	if ip == nil {
+		c.log.Infof("Failed to parse service network cidr %s, skipping", networks[0])
 		return
 	}
-	ip4[3] = 10 // .10 is the conflicting address
+
+	ip[len(ip)-1] = 10 // .10 or :a is the conflicting address
 
 	for i := 0; i < maxDNSServiceIPAttempts; i++ {
 		svs, err := c.kc.ListServices("")
@@ -269,17 +299,17 @@ func (c *controller) HackDNSAddressConflict(wg *sync.WaitGroup) {
 			time.Sleep(DNSAddressRetryInterval)
 			continue
 		}
-		s := c.findServiceByIP(ip4.String(), &svs.Items)
+		s := c.findServiceByIP(ip.String(), &svs.Items)
 		if s == nil {
-			c.log.Infof("No service found with IP %s, attempt %d/%d", ip4, i+1, maxDNSServiceIPAttempts)
+			c.log.Infof("No service found with IP %s, attempt %d/%d", ip, i+1, maxDNSServiceIPAttempts)
 			time.Sleep(DNSAddressRetryInterval)
 			continue
 		}
 		if s.Name == dnsServiceName && s.Namespace == dnsServiceNamespace {
-			c.log.Infof("Service %s has successfully taken IP %s", dnsServiceName, ip4)
+			c.log.Infof("Service %s has successfully taken IP %s", dnsServiceName, ip)
 			break
 		}
-		c.log.Warnf("Deleting service %s in namespace %s whose IP %s conflicts with %s", s.Name, s.Namespace, ip4, dnsServiceName)
+		c.log.Warnf("Deleting service %s in namespace %s whose IP %s conflicts with %s", s.Name, s.Namespace, ip, dnsServiceName)
 		if err := c.killConflictingService(s); err != nil {
 			c.log.WithError(err).Warnf("Failed to delete service %s in namespace %s", s.Name, s.Namespace)
 			continue
@@ -383,7 +413,7 @@ func (c controller) PostInstallConfigs(ctx context.Context, wg *sync.WaitGroup)
 	}()
 	err := utils.WaitForPredicateWithContext(ctx, LongWaitTimeout, GeneralWaitInterval, func() bool {
 		ctxReq := utils.GenerateRequestContext()
-		cluster, err := c.ic.GetCluster(ctx)
+		cluster, err := c.ic.GetCluster(ctx, false)
 		if err != nil {
 			utils.RequestIDLogger(ctxReq, c.log).WithError(err).Errorf("Failed to get cluster %s from assisted-service", c.ClusterID)
 			return false
@@ -413,25 +443,8 @@ func (c controller) PostInstallConfigs(ctx context.Context, wg *sync.WaitGroup)
 func (c controller) postInstallConfigs(ctx context.Context) error {
 	var err error
 
-	c.log.Infof("Waiting for cluster version operator: %t", c.WaitForClusterVersion)
-
-	if c.WaitForClusterVersion {
-		if err = c.waitingForClusterVersion(ctx); err != nil {
-			return errors.Wrapf(err, "Timeout while waiting for cluster version to be available")
-		}
-	}
-
-	// Unlabel run-level from assisted-installer namespace after the installation.
-	// Keeping the `run-level` label represents a security risk as it overwrites the SecurityContext configurations
-	// used for applications deployed in this namespace.
-	data := []byte(`{"metadata":{"labels":{"$patch": "delete", "openshift.io/run-level":"0"}}}`)
-	c.log.Infof("Removing run-level label from %s namespace", c.ControllerConfig.Namespace)
-	err = c.kc.PatchNamespace(c.ControllerConfig.Namespace, data)
-	if err != nil {
-		// It is a conscious decision not to fail an installation if for any reason patching the namespace
-		// in order to remove the `run-level` label has failed. This will be redesigned in the next release
-		// so that the `run-level` label is not created in the first place.
-		c.log.Warn("Failed to unlabel AI namespace after the installation.")
+	if err = c.waitingForClusterOperators(ctx); err != nil {
+		return errors.Wrapf(err, "Timeout while waiting for cluster operators to be available")
 	}
 
 	err = utils.WaitForPredicateWithContext(ctx, WaitTimeout, GeneralWaitInterval, c.addRouterCAToClusterCA)
@@ -451,20 +464,55 @@ func (c controller) postInstallConfigs(ctx context.Context) error {
 		c.log.Infof("Skipping etcd unpatch for cluster version %s", c.ControllerConfig.OpenshiftVersion)
 	}
 
-	if err = utils.WaitForPredicateWithContext(ctx, WaitTimeout, GeneralWaitInterval, c.validateConsoleAvailability); err != nil {
-		return errors.Wrapf(err, "Timeout while waiting for console to become available")
+	// Wait for OLM operators
+	if err = c.waitForOLMOperators(ctx); err != nil {
+		return errors.Wrapf(err, "Error while initializing OLM operators")
+	}
+
+	return nil
+}
+
+func (c controller) waitForOLMOperators(ctx context.Context) error {
+	var operators []models.MonitoredOperator
+	var err error
+
+	// Get the monitored operators:
+	err = utils.Retry(maxFetchAttempts, FetchRetryInterval, c.log, func() error {
+		operators, err = c.ic.GetClusterMonitoredOLMOperators(context.TODO(), c.ClusterID, c.OpenshiftVersion)
+		if err != nil {
+			return errors.Wrapf(err, "Error while fetch the monitored operators from assisted-service.")
+		}
+		return nil
+	})
+	if err != nil {
+		return errors.Wrapf(err, "Failed to fetch monitored operators")
+	}
+	if len(operators) == 0 {
+		c.log.Info("No OLM operators found.")
+		return nil
+	}
+
+	// Get maximum wait timeout for OLM operators:
+	waitTimeout := c.getMaximumOLMTimeout(operators)
+	c.log.Infof("OLM operators %v wait timeout %v", waitTimeout, operators)
+
+	// Wait for the CSV state of the OLM operators, before applying OLM CRs
+	err = utils.WaitForPredicateParamsWithContext(ctx, waitTimeout, GeneralWaitInterval, c.waitForCSVBeCreated, operators)
+	if err != nil {
+		// We continue in case of failure, because we want to try to apply manifest at least for operators which are ready.
+		c.log.WithError(err).Warnf("Failed to wait for some of the OLM operators to be initilized")
 	}
 
 	// Apply post install manifests
-	err = utils.WaitForPredicateWithContext(ctx, retryPostManifestTimeout, GeneralWaitInterval, c.applyPostInstallManifests)
+	err = utils.WaitForPredicateParamsWithContext(ctx, retryPostManifestTimeout, GeneralWaitInterval, c.applyPostInstallManifests, operators)
 	if err != nil {
 		return errors.Wrapf(err, "Failed to apply post manifests")
 	}
 
-	if err != c.waitForOLMOperators(ctx) {
+	if err != c.waitForCSV(ctx, waitTimeout) {
 		// In case the timeout occur, we have to update the pending OLM operators to failed state,
 		// so the assisted-service can update the cluster state to completed.
-		if err = c.updatePendingOLMOperators(); err != nil {
+		if err = c.updatePendingOLMOperators(ctx); err != nil {
 			return errors.Errorf("Timeout while waiting for some of the operators and not able to update its state")
 		}
 		return errors.Wrapf(err, "Timeout while waiting for OLM operators be installed")
@@ -473,7 +521,32 @@ func (c controller) postInstallConfigs(ctx context.Context) error {
 	return nil
 }
 
-func (c controller) applyPostInstallManifests() bool {
+func (c controller) getReadyOperators(operators []models.MonitoredOperator) ([]string, []models.MonitoredOperator, error) {
+	var readyOperators []string
+	for index := range operators {
+		handler := NewClusterServiceVersionHandler(c.kc, &operators[index], c.Status)
+		if handler.IsInitialized() {
+			readyOperators = append(readyOperators, handler.GetName())
+		}
+	}
+	return readyOperators, operators, nil
+}
+
+func (c controller) waitForCSVBeCreated(arg interface{}) bool {
+	operators := arg.([]models.MonitoredOperator)
+	readyOperators, operators, err := c.getReadyOperators(operators)
+	if err != nil {
+		c.log.WithError(err).Warn("Error while fetch the operators state.")
+		return false
+	}
+	if len(operators) == len(readyOperators) {
+		return true
+	}
+
+	return false
+}
+
+func (c controller) applyPostInstallManifests(arg interface{}) bool {
 	ctx := utils.GenerateRequestContext()
 	tempDir, err := ioutil.TempDir("", "controller-custom-manifests-")
 	if err != nil {
@@ -493,11 +566,69 @@ func (c controller) applyPostInstallManifests() bool {
 		return false
 	}
 
-	err = c.ops.CreateManifests(kubeconfigName, customManifestPath)
+	// Unmarshall the content of the operators manifests:
+	var manifests []manifest
+	data, err := ioutil.ReadFile(customManifestPath)
 	if err != nil {
-		c.log.WithError(err).Error("Failed to apply manifest file.")
+		c.log.WithError(err).Errorf("Failed to read the custom manifests file.")
 		return false
 	}
+	if err = json.Unmarshal(data, &manifests); err != nil {
+		c.log.WithError(err).Errorf("Failed to unmarshall custom manifest file content %s.", data)
+		return false
+	}
+
+	// Create the manifests of the opreators, which are properly initialized:
+	readyOperators, _, err := c.getReadyOperators(arg.([]models.MonitoredOperator))
+	if err != nil {
+		c.log.WithError(err).Errorf("Failed to fetch operators from assisted-service")
+		return false
+	}
+
+	c.log.Infof("Ready operators to be applied: %v", readyOperators)
+
+	for _, manifest := range manifests {
+		v1, err := version.NewVersion(c.OpenshiftVersion)
+		if err != nil {
+			return false
+		}
+		constraints, err := version.NewConstraint(">= 4.8, < 4.9")
+		if err != nil {
+			return false
+		}
+
+		// renaming odf if ocs as manifest is applied based on the ready operators list
+		if manifest.Name == "odf" && constraints.Check(v1) {
+			manifest.Name = "ocs"
+		}
+		c.log.Infof("Applying manifest %s: %s", manifest.Name, manifest.Content)
+
+		// Check if the operator is properly initialized by CSV:
+		if !func() bool {
+			for _, readyOperator := range readyOperators {
+				if readyOperator == manifest.Name {
+					return true
+				}
+			}
+			return false
+		}() {
+			continue
+		}
+
+		content, err := base64.StdEncoding.DecodeString(manifest.Content)
+		if err != nil {
+			c.log.WithError(err).Errorf("Failed to decode content of operator CR %s.", manifest.Name)
+			return false
+		}
+
+		err = c.ops.CreateManifests(kubeconfigName, content)
+		if err != nil {
+			c.log.WithError(err).Error("Failed to apply manifest file.")
+			return false
+		}
+
+		c.log.Infof("Manifest %s applied.", manifest.Name)
+	}
 
 	return true
 }
@@ -749,7 +880,7 @@ func (c controller) addRouterCAToClusterCA() bool {
 
 }
 
-func (c controller) getMaximumOLMTimeout(operators []*models.MonitoredOperator) time.Duration {
+func (c controller) getMaximumOLMTimeout(operators []models.MonitoredOperator) time.Duration {
 	timeout := WaitTimeout.Seconds()
 	for _, operator := range operators {
 		timeout = math.Max(float64(operator.TimeoutSeconds), timeout)
@@ -760,7 +891,7 @@ func (c controller) getMaximumOLMTimeout(operators []*models.MonitoredOperator)
 
 func (c controller) getProgressingOLMOperators() ([]*models.MonitoredOperator, error) {
 	ret := make([]*models.MonitoredOperator, 0)
-	operators, err := c.ic.GetClusterMonitoredOLMOperators(context.TODO(), c.ClusterID)
+	operators, err := c.ic.GetClusterMonitoredOLMOperators(context.TODO(), c.ClusterID, c.OpenshiftVersion)
 	if err != nil {
 		c.log.WithError(err).Warningf("Failed to connect to assisted service")
 		return ret, err
@@ -773,9 +904,8 @@ func (c controller) getProgressingOLMOperators() ([]*models.MonitoredOperator, e
 	return ret, nil
 }
 
-func (c controller) updatePendingOLMOperators() error {
+func (c controller) updatePendingOLMOperators(ctx context.Context) error {
 	c.log.Infof("Updating pending OLM operators")
-	ctx := utils.GenerateRequestContext()
 	operators, err := c.getProgressingOLMOperators()
 	if err != nil {
 		return err
@@ -791,8 +921,8 @@ func (c controller) updatePendingOLMOperators() error {
 	return nil
 }
 
-// waitForOLMOperators wait until all OLM monitored operators are available or failed.
-func (c controller) waitForOLMOperators(ctx context.Context) error {
+// waitForCSV wait until all OLM monitored operators are available or failed.
+func (c controller) waitForCSV(ctx context.Context, waitTimeout time.Duration) error {
 	operators, err := c.getProgressingOLMOperators()
 	if err != nil {
 		return err
@@ -806,7 +936,6 @@ func (c controller) waitForOLMOperators(ctx context.Context) error {
 	for index := range operators {
 		handlers[operators[index].Name] = NewClusterServiceVersionHandler(c.kc, operators[index], c.Status)
 	}
-
 	areOLMOperatorsAvailable := func() bool {
 		if len(handlers) == 0 {
 			return true
@@ -820,26 +949,26 @@ func (c controller) waitForOLMOperators(ctx context.Context) error {
 		return false
 	}
 
-	waitTimeout := c.getMaximumOLMTimeout(operators)
-	c.log.Infof("Waiting for OLM operators for %v", waitTimeout)
 	return utils.WaitForPredicateWithContext(ctx, waitTimeout, GeneralWaitInterval, areOLMOperatorsAvailable)
 }
 
-// validateConsoleAvailability checks if the console operator is available
-func (c controller) validateConsoleAvailability() bool {
-	return c.isOperatorAvailable(NewClusterOperatorHandler(c.kc, consoleOperatorName))
-}
-
-// waitingForClusterVersion checks the Cluster Version Operator availability in the
-// new OCP cluster. A success would be announced only when the service acknowledges
-// the CVO availability, in order to avoid unsycned scenarios.
-// In case cvo changes it message we will update timer but we want to have maximum timeout
-// for this context with timeout is used
-func (c controller) waitingForClusterVersion(ctx context.Context) error {
+// waitingForClusterOperators checks Console operator and the Cluster Version Operator availability in the
+// new OCP cluster in parallel.
+// A success would be announced only when the service acknowledges the operators availability,
+// in order to avoid unsycned scenarios.
+func (c controller) waitingForClusterOperators(ctx context.Context) error {
+	// In case cvo changes it message we will update timer but we want to have maximum timeout
+	// for this context with timeout is used
 	ctxWithTimeout, cancel := context.WithTimeout(ctx, CVOMaxTimeout)
 	defer cancel()
 	isClusterVersionAvailable := func(timer *time.Timer) bool {
-		return c.isOperatorAvailable(NewClusterVersionHandler(c.kc, timer))
+		result := c.isOperatorAvailable(NewClusterOperatorHandler(c.kc, consoleOperatorName))
+
+		if c.WaitForClusterVersion {
+			result = c.isOperatorAvailable(NewClusterVersionHandler(c.kc, timer)) && result
+		}
+
+		return result
 	}
 	return utils.WaitForPredicateWithTimer(ctxWithTimeout, WaitTimeout, GeneralProgressUpdateInt, isClusterVersionAvailable)
 }
@@ -882,8 +1011,14 @@ func (c controller) uploadSummaryLogs(podName string, namespace string, sinceSec
 	var ok bool = true
 	ctx := utils.GenerateRequestContext()
 
+	// Send upload operator logs before must-gather
 	c.logClusterOperatorsStatus()
 	if c.Status.HasError() || c.Status.HasOperatorError() {
+		c.log.Infof("Uploading cluster operator status logs before must-gather")
+		err := common.UploadPodLogs(c.kc, c.ic, c.ClusterID, podName, c.Namespace, controllerLogsSecondsAgo, c.log)
+		if err != nil {
+			c.log.WithError(err).Warnf("Failed to upload controller logs")
+		}
 		c.log.Infof("Uploading oc must-gather logs")
 		images := c.parseMustGatherImages()
 		if tarfile, err := c.collectMustGatherLogs(ctx, images...); err == nil {
@@ -973,7 +1108,7 @@ func (c controller) parseMustGatherImages() []string {
 func (c controller) downloadKubeconfigNoingress(ctx context.Context, dir string) (string, error) {
 	// Download kubeconfig file
 	kubeconfigPath := path.Join(dir, kubeconfigFileName)
-	err := c.ic.DownloadFile(ctx, kubeconfigFileName, kubeconfigPath)
+	err := c.ic.DownloadClusterCredentials(ctx, kubeconfigFileName, kubeconfigPath)
 	if err != nil {
 		c.log.Errorf("Failed to download noingress kubeconfig %v\n", err)
 		return "", err
@@ -1063,7 +1198,7 @@ func (c *controller) UploadLogs(ctx context.Context, wg *sync.WaitGroup) {
 func (c controller) SetReadyState() {
 	c.log.Infof("Start waiting to be ready")
 	_ = utils.WaitForPredicate(WaitTimeout, 1*time.Second, func() bool {
-		_, err := c.ic.GetCluster(context.TODO())
+		_, err := c.ic.GetCluster(context.TODO(), false)
 		if err != nil {
 			c.log.WithError(err).Warningf("Failed to connect to assisted service")
 			return false
