@@ -4,9 +4,9 @@ import (
 	"context"
 	"fmt"
 	"path/filepath"
-	"strings"
 	"time"
 
+	"github.com/go-openapi/swag"
 	"github.com/google/uuid"
 	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
@@ -42,6 +42,8 @@ var generalWaitInterval = 5 * time.Second
 
 // Installer will run the install operations on the node
 type Installer interface {
+	// FormatDisks formats all disks that have been configured to be formatted
+	FormatDisks() error
 	InstallNode() error
 	UpdateHostInstallProgress(newStage models.HostStage, info string)
 }
@@ -66,6 +68,16 @@ func NewAssistedInstaller(log *logrus.Logger, cfg config.Config, ops ops.Ops, ic
 	}
 }
 
+func (i *installer) FormatDisks() {
+	for _, diskToFormat := range i.Config.DisksToFormat {
+		if err := i.ops.FormatDisk(diskToFormat); err != nil {
+			// This is best effort - keep trying to format other disks
+			// and go on with the installation, log a warning
+			i.log.Warnf("Failed to format disk %s, err %s", diskToFormat, err)
+		}
+	}
+}
+
 func (i *installer) InstallNode() error {
 	i.log.Infof("Installing node with role: %s", i.Config.Role)
 
@@ -133,10 +145,15 @@ func (i *installer) InstallNode() error {
 		}
 		i.log.Info("Setting bootstrap node new role to master")
 
+	} else if i.Config.Role == string(models.HostRoleWorker) {
+		// Wait for 2 masters to be ready before rebooting
+		if err = i.workerWaitFor2ReadyMasters(ctx); err != nil {
+			return err
+		}
 	}
 	//upload host logs and report log status before reboot
 	i.log.Infof("Uploading logs and reporting status before rebooting the node %s for cluster %s", i.Config.HostID, i.Config.ClusterID)
-	i.inventoryClient.HostLogProgressReport(ctx, i.Config.ClusterID, i.Config.HostID, models.LogsStateRequested)
+	i.inventoryClient.HostLogProgressReport(ctx, i.Config.InfraEnvID, i.Config.HostID, models.LogsStateRequested)
 	_, err = i.ops.UploadInstallationLogs(isBootstrap || i.HighAvailabilityMode == models.ClusterHighAvailabilityModeNone)
 	if err != nil {
 		i.log.Errorf("upload installation logs %s", err)
@@ -152,6 +169,10 @@ func (i *installer) InstallNode() error {
 
 //updateSingleNodeIgnition will download the host ignition config and add the files under storage
 func (i *installer) updateSingleNodeIgnition(singleNodeIgnitionPath string) error {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
+	}
+
 	hostIgnitionPath, err := i.downloadHostIgnition()
 	if err != nil {
 		return err
@@ -195,6 +216,12 @@ func (i *installer) writeImageToDisk(ignitionPath string) error {
 
 func (i *installer) startBootstrap() error {
 	i.log.Infof("Running bootstrap")
+	// This is required for the log collection command to work since it will try to mount this directory
+	// This directory is also required by `generateSshKeyPair` as it will place the key there
+	if err := i.ops.Mkdir(sshDir); err != nil {
+		i.log.WithError(err).Error("Failed to create SSH dir")
+		return err
+	}
 	ignitionFileName := "bootstrap.ign"
 	ignitionPath, err := i.getFileFromService(ignitionFileName)
 	if err != nil {
@@ -264,6 +291,10 @@ func (i *installer) startBootstrap() error {
 }
 
 func (i *installer) extractIgnitionToFS(ignitionPath string) (err error) {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
+	}
+
 	mcoImage := i.MCOImage
 
 	i.log.Infof("Extracting ignition to disk using %s mcoImage", mcoImage)
@@ -288,11 +319,11 @@ func (i *installer) extractIgnitionToFS(ignitionPath string) (err error) {
 }
 
 func (i *installer) generateSshKeyPair() error {
-	i.log.Info("Generating new SSH key pair")
-	if err := i.ops.Mkdir(sshDir); err != nil {
-		i.log.WithError(err).Error("Failed to create SSH dir")
-		return err
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
 	}
+
+	i.log.Info("Generating new SSH key pair")
 	if _, err := i.ops.ExecPrivilegeCommand(utils.NewLogWriter(i.log), "ssh-keygen", "-q", "-f", sshKeyPath, "-N", ""); err != nil {
 		i.log.WithError(err).Error("Failed to generate SSH key pair")
 		return err
@@ -319,7 +350,7 @@ func (i *installer) downloadHostIgnition() (string, error) {
 	log.Infof("Getting %s file", filename)
 
 	dest := filepath.Join(InstallDir, filename)
-	err := i.inventoryClient.DownloadHostIgnition(ctx, i.Config.HostID, dest)
+	err := i.inventoryClient.DownloadHostIgnition(ctx, i.Config.InfraEnvID, i.Config.HostID, dest)
 	if err != nil {
 		log.Errorf("Failed to fetch file (%s) from server. err: %s", filename, err)
 	}
@@ -378,6 +409,40 @@ func (i *installer) waitForControlPlane(ctx context.Context) error {
 	return nil
 }
 
+func numDone(hosts models.HostList) int {
+	numDone := 0
+	for _, h := range hosts {
+		if h.Progress.CurrentStage == models.HostStageDone {
+			numDone++
+		}
+	}
+	return numDone
+}
+
+func (i *installer) workerWaitFor2ReadyMasters(ctx context.Context) error {
+	i.log.Info("Waiting for 2 ready masters")
+	i.UpdateHostInstallProgress(models.HostStageWaitingForControlPlane, "")
+	cluster, err := i.inventoryClient.GetCluster(ctx, false)
+	if err != nil {
+		i.log.WithError(err).Errorf("Getting cluster %s", i.ClusterID)
+		return err
+	}
+	if swag.StringValue(cluster.Kind) == models.ClusterKindAddHostsCluster {
+		return nil
+	}
+	for {
+		hosts, err := i.inventoryClient.ListsHostsForRole(ctx, string(models.HostRoleMaster))
+		if err != nil {
+			i.log.WithError(err).Errorf("Getting cluster %s hosts", i.ClusterID)
+			return err
+		}
+		if numDone(hosts) >= minMasterNodes {
+			return nil
+		}
+		time.Sleep(generalWaitInterval)
+	}
+}
+
 func (i *installer) shouldControlPlaneReplicasPatchApplied(kc k8s_client.K8SClient) (bool, error) {
 	controlPlanePatchRequired, err := utils.IsVersionLessThan47(i.Config.OpenshiftVersion)
 	if err != nil {
@@ -447,7 +512,7 @@ func (i *installer) UpdateHostInstallProgress(newStage models.HostStage, info st
 	log := utils.RequestIDLogger(ctx, i.log)
 	log.Infof("Updating node installation stage: %s - %s", newStage, info)
 	if i.HostID != "" {
-		if err := i.inventoryClient.UpdateHostInstallProgress(ctx, i.HostID, newStage, info); err != nil {
+		if err := i.inventoryClient.UpdateHostInstallProgress(ctx, i.Config.InfraEnvID, i.Config.HostID, newStage, info); err != nil {
 			log.Errorf("Failed to update node installation stage, %s", err)
 		}
 	}
@@ -600,6 +665,8 @@ func (i *installer) getInventoryHostsMap(hostsMap map[string]inventory_client.Ho
 
 func (i *installer) updateReadyMasters(nodes *v1.NodeList, readyMasters *[]string, inventoryHostsMap map[string]inventory_client.HostData) error {
 	nodeNameAndCondition := map[string][]v1.NodeCondition{}
+	knownIpAddresses := common.BuildHostsMapIPAddressBased(inventoryHostsMap)
+
 	for _, node := range nodes.Items {
 		nodeNameAndCondition[node.Name] = node.Status.Conditions
 		if common.IsK8sNodeIsReady(node) && !funk.ContainsString(*readyMasters, node.Name) {
@@ -607,12 +674,13 @@ func (i *installer) updateReadyMasters(nodes *v1.NodeList, readyMasters *[]strin
 			log := utils.RequestIDLogger(ctx, i.log)
 			log.Infof("Found a new ready master node %s with id %s", node.Name, node.Status.NodeInfo.SystemUUID)
 			*readyMasters = append(*readyMasters, node.Name)
-			host, ok := inventoryHostsMap[strings.ToLower(node.Name)]
+
+			host, ok := common.HostMatchByNameOrIPAddress(node, inventoryHostsMap, knownIpAddresses)
 			if !ok {
 				return fmt.Errorf("Node %s is not in inventory hosts", node.Name)
 			}
 			ctx = utils.GenerateRequestContext()
-			if err := i.inventoryClient.UpdateHostInstallProgress(ctx, host.Host.ID.String(), models.HostStageJoined, ""); err != nil {
+			if err := i.inventoryClient.UpdateHostInstallProgress(ctx, host.Host.InfraEnvID.String(), host.Host.ID.String(), models.HostStageJoined, ""); err != nil {
 				utils.RequestIDLogger(ctx, i.log).Errorf("Failed to update node installation status, %s", err)
 			}
 		}
@@ -623,24 +691,40 @@ func (i *installer) updateReadyMasters(nodes *v1.NodeList, readyMasters *[]strin
 }
 
 func (i *installer) cleanupInstallDevice() error {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
+	}
+
 	vgName, err := i.ops.GetVGByPV(i.Device)
 	if err != nil {
 		return err
 	}
 
-	if vgName == "" {
-		i.log.Infof("No VG/LVM was found on device %s, no need to clean", i.Device)
-		return nil
+	if vgName != "" {
+		i.log.Infof("A virtual group was detected on the installation device (%s) - cleaning", i.Device)
+		err = i.ops.RemoveVG(vgName)
+
+		if err != nil {
+			return err
+		}
+
+		err = i.ops.RemovePV(i.Device)
+
+		if err != nil {
+			return err
+		}
 	}
 
-	err = i.ops.RemoveVG(vgName)
-	if err != nil {
-		return err
+	if i.ops.IsRaidDevice(i.Device) {
+		i.log.Infof("A raid was detected on the installation device (%s) - cleaning", i.Device)
+		err = i.ops.CleanRaidDevice(i.Device)
+
+		if err != nil {
+			return err
+		}
 	}
 
-	_ = i.ops.Wipefs(i.Device)
-
-	return i.ops.RemovePV(i.Device)
+	return i.ops.Wipefs(i.Device)
 }
 
 func (i *installer) verifyHostCanMoveToConfigurationStatus(inventoryHostsMapWithIp map[string]inventory_client.HostData) {
@@ -716,6 +800,10 @@ func (i *installer) createSingleNodeMasterIgnition() (string, error) {
 }
 
 func (i *installer) checkLocalhostName() error {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
+	}
+
 	i.log.Infof("Start checking localhostname")
 	hostname, err := i.ops.GetHostname()
 	if err != nil {
