@@ -43,6 +43,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 		mockIgnition       *ignition.MockIgnition
 		installerObj       *installer
 		hostId             = "host-id"
+		infraEnvId         = "infra-env-id"
 		bootstrapIgn       = "bootstrap.ign"
 		openShiftVersion   = "4.7"
 		inventoryNamesHost map[string]inventory_client.HostData
@@ -65,8 +66,8 @@ var _ = Describe("installer HostRoleMaster role", func() {
 	downloadFileSuccess := func(fileName string) {
 		mockbmclient.EXPECT().DownloadFile(gomock.Any(), fileName, filepath.Join(InstallDir, fileName)).Return(nil).Times(1)
 	}
-	downloadHostIgnitionSuccess := func(hostID string, fileName string) {
-		mockbmclient.EXPECT().DownloadHostIgnition(gomock.Any(), hostID, filepath.Join(InstallDir, fileName)).Return(nil).Times(1)
+	downloadHostIgnitionSuccess := func(infraEnvID string, hostID string, fileName string) {
+		mockbmclient.EXPECT().DownloadHostIgnition(gomock.Any(), infraEnvID, hostID, filepath.Join(InstallDir, fileName)).Return(nil).Times(1)
 	}
 
 	reportLogProgressSuccess := func() {
@@ -84,14 +85,16 @@ var _ = Describe("installer HostRoleMaster role", func() {
 
 	cleanInstallDevice := func() {
 		mockops.EXPECT().GetVGByPV(device).Return("", nil).Times(1)
+		mockops.EXPECT().IsRaidDevice(device).Return(false).Times(1)
+		mockops.EXPECT().Wipefs(device).Return(nil).Times(1)
 	}
 
 	updateProgressSuccess := func(stages [][]string) {
 		for _, stage := range stages {
 			if len(stage) == 2 {
-				mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), hostId, models.HostStage(stage[0]), stage[1]).Return(nil).Times(1)
+				mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), infraEnvId, hostId, models.HostStage(stage[0]), stage[1]).Return(nil).Times(1)
 			} else {
-				mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), hostId, models.HostStage(stage[0]), "").Return(nil).Times(1)
+				mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), infraEnvId, hostId, models.HostStage(stage[0]), "").Return(nil).Times(1)
 			}
 		}
 	}
@@ -109,7 +112,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 	}
 
 	waitForControllerSuccessfully := func(clusterId string) {
-		mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), hostId, models.HostStageWaitingForController, "waiting for controller pod ready event").Return(nil).Times(1)
+		mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), infraEnvId, hostId, models.HostStageWaitingForController, "waiting for controller pod ready event").Return(nil).Times(1)
 		mockk8sclient.EXPECT().GetPods("assisted-installer", gomock.Any(), "").Return([]v1.Pod{{TypeMeta: metav1.TypeMeta{},
 			ObjectMeta: metav1.ObjectMeta{Name: common.AssistedControllerPrefix + "aasdasd"},
 			Status:     v1.PodStatus{Phase: "Running"}}}, nil).Times(1)
@@ -133,12 +136,13 @@ var _ = Describe("installer HostRoleMaster role", func() {
 		mockbmclient = inventory_client.NewMockInventoryClient(ctrl)
 		mockk8sclient = k8s_client.NewMockK8SClient(ctrl)
 		mockIgnition = ignition.NewMockIgnition(ctrl)
+		nodesInfraEnvId := strfmt.UUID("7916fa89-ea7a-443e-a862-b3e930309f50")
 		node0Id := strfmt.UUID("7916fa89-ea7a-443e-a862-b3e930309f65")
 		node1Id := strfmt.UUID("eb82821f-bf21-4614-9a3b-ecb07929f238")
 		node2Id := strfmt.UUID("b898d516-3e16-49d0-86a5-0ad5bd04e3ed")
-		inventoryNamesHost = map[string]inventory_client.HostData{"node0": {Host: &models.Host{ID: &node0Id}, IPs: []string{"192.168.126.10"}},
-			"node1": {Host: &models.Host{ID: &node1Id}, IPs: []string{"192.168.126.11"}},
-			"node2": {Host: &models.Host{ID: &node2Id}, IPs: []string{"192.168.126.12"}}}
+		inventoryNamesHost = map[string]inventory_client.HostData{"node0": {Host: &models.Host{InfraEnvID: nodesInfraEnvId, ID: &node0Id}, IPs: []string{"192.168.126.10"}},
+			"node1": {Host: &models.Host{InfraEnvID: nodesInfraEnvId, ID: &node1Id}, IPs: []string{"192.168.126.11"}},
+			"node2": {Host: &models.Host{InfraEnvID: nodesInfraEnvId, ID: &node2Id}, IPs: []string{"192.168.126.12"}}}
 	})
 	k8sBuilder := func(configPath string, logger *logrus.Logger) (k8s_client.K8SClient, error) {
 		return mockk8sclient, nil
@@ -148,6 +152,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 
 		conf := config.Config{Role: string(models.HostRoleBootstrap),
 			ClusterID:        "cluster-id",
+			InfraEnvID:       "infra-env-id",
 			HostID:           "host-id",
 			Device:           "/dev/vda",
 			URL:              "https://assisted-service.com:80",
@@ -194,11 +199,11 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			mockk8sclient.EXPECT().ListMasterNodes().Return(GetKubeNodes(map[string]string{}), nil).Times(1)
 			kubeNamesIds = map[string]string{"node0": "7916fa89-ea7a-443e-a862-b3e930309f65"}
 			mockk8sclient.EXPECT().ListMasterNodes().Return(GetKubeNodes(kubeNamesIds), nil).Times(1)
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), inventoryNamesHost["node0"].Host.ID.String(), models.HostStageJoined, "").Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), inventoryNamesHost["node0"].Host.InfraEnvID.String(), inventoryNamesHost["node0"].Host.ID.String(), models.HostStageJoined, "").Times(1)
 			kubeNamesIds = map[string]string{"node0": "7916fa89-ea7a-443e-a862-b3e930309f65",
 				"node1": "eb82821f-bf21-4614-9a3b-ecb07929f238"}
 			mockk8sclient.EXPECT().ListMasterNodes().Return(GetKubeNodes(kubeNamesIds), nil).Times(1)
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), inventoryNamesHost["node1"].Host.ID.String(), models.HostStageJoined, "").Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), inventoryNamesHost["node1"].Host.InfraEnvID.String(), inventoryNamesHost["node1"].Host.ID.String(), models.HostStageJoined, "").Times(1)
 		}
 		getNetworkTypeSuccessOpenshiftSDN := func() {
 			mockk8sclient.EXPECT().GetNetworkType().Return("OpenshiftSDN", nil).Times(2)
@@ -219,7 +224,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			mockops.EXPECT().PrepareController().Return(nil).Times(1)
 		}
 		waitForBootkubeSuccess := func() {
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), hostId, models.HostStageWaitingForBootkube, "").Return(nil).Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), infraEnvId, hostId, models.HostStageWaitingForBootkube, "").Return(nil).Times(1)
 			mockops.EXPECT().ExecPrivilegeCommand(gomock.Any(), "stat", "/opt/openshift/.bootkube.done").Return("OK", nil).Times(1)
 		}
 		bootkubeStatusSuccess := func() {
@@ -230,7 +235,6 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			mockops.EXPECT().ExtractFromIgnition(filepath.Join(InstallDir, bootstrapIgn), dockerConfigFile).Return(nil).Times(1)
 		}
 		generateSshKeyPairSuccess := func() {
-			mkdirSuccess(sshDir)
 			mockops.EXPECT().ExecPrivilegeCommand(gomock.Any(), "ssh-keygen", "-q", "-f", sshKeyPath, "-N", "").Return("OK", nil).Times(1)
 		}
 		createOpenshiftSshManifestSuccess := func() {
@@ -239,6 +243,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 
 		bootstrapSetup := func() {
 			cleanInstallDevice()
+			mkdirSuccess(sshDir)
 			mkdirSuccess(InstallDir)
 			downloadFileSuccess(bootstrapIgn)
 			extractSecretFromIgnitionSuccess()
@@ -276,7 +281,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 					resolvConfSuccess()
 					waitForControllerSuccessfully(conf.ClusterID)
 					//HostRoleMaster flow:
-					downloadHostIgnitionSuccess(hostId, "master-host-id.ign")
+					downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
 					writeToDiskSuccess(gomock.Any())
 					reportLogProgressSuccess()
 					setBootOrderSuccess(gomock.Any())
@@ -309,7 +314,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 					resolvConfSuccess()
 					waitForControllerSuccessfully(conf.ClusterID)
 					//HostRoleMaster flow:
-					downloadHostIgnitionSuccess(hostId, "master-host-id.ign")
+					downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
 					writeToDiskSuccess(gomock.Any())
 					setBootOrderSuccess(gomock.Any())
 					uploadLogsSuccess(true)
@@ -327,6 +332,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			})
 			cleanInstallDevice()
 			mkdirSuccess(InstallDir)
+			mkdirSuccess(sshDir)
 			downloadFileSuccess(bootstrapIgn)
 			extractSecretFromIgnitionSuccess()
 			extractIgnitionToFS("Success", nil)
@@ -334,7 +340,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			err := fmt.Errorf("generate SSH keys failed")
 			mockops.EXPECT().CreateOpenshiftSshManifest(assistedInstallerSshManifest, sshManifestTmpl, sshPubKeyPath).Return(err).Times(1)
 			//HostRoleMaster flow:
-			downloadHostIgnitionSuccess(hostId, "master-host-id.ign")
+			downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
 			writeToDiskSuccess(gomock.Any())
 			setBootOrderSuccess(gomock.Any())
 			ret := installerObj.InstallNode()
@@ -359,7 +365,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			resolvConfSuccess()
 			waitForControllerSuccessfully(conf.ClusterID)
 			//HostRoleMaster flow:
-			downloadHostIgnitionSuccess(hostId, "master-host-id.ign")
+			downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
 			writeToDiskSuccess(gomock.Any())
 			setBootOrderSuccess(gomock.Any())
 			uploadLogsSuccess(true)
@@ -375,8 +381,9 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			})
 			cleanInstallDevice()
 			mkdirSuccess(InstallDir)
+			mkdirSuccess(sshDir)
 			downloadFileSuccess(bootstrapIgn)
-			downloadHostIgnitionSuccess(hostId, "master-host-id.ign")
+			downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
 			writeToDiskSuccess(gomock.Any())
 			setBootOrderSuccess(gomock.Any())
 			extractSecretFromIgnitionSuccess()
@@ -397,7 +404,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			err := fmt.Errorf("Failed to restart NetworkManager")
 			restartNetworkManager(err)
 			//HostRoleMaster flow:
-			downloadHostIgnitionSuccess(hostId, "master-host-id.ign")
+			downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
 			writeToDiskSuccess(gomock.Any())
 			setBootOrderSuccess(gomock.Any())
 			ret := installerObj.InstallNode()
@@ -408,6 +415,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 
 		conf := config.Config{Role: string(models.HostRoleBootstrap),
 			ClusterID:        "cluster-id",
+			InfraEnvID:       "infra-env-id",
 			HostID:           "host-id",
 			Device:           "/dev/vda",
 			URL:              "https://assisted-service.com:80",
@@ -426,7 +434,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 
 		It("waitForController reload get pods fails then succeeds", func() {
 			reportLogProgressSuccess()
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), hostId, models.HostStageWaitingForController, "waiting for controller pod ready event").Return(nil).Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), infraEnvId, hostId, models.HostStageWaitingForController, "waiting for controller pod ready event").Return(nil).Times(1)
 			mockk8sclient.EXPECT().GetPods("assisted-installer", gomock.Any(), "").Return(nil, fmt.Errorf("dummy")).Times(1)
 			mockk8sclient.EXPECT().ListEvents(assistedControllerNamespace).Return(&events, nil).Times(1)
 			err := installerObj.waitForController(mockk8sclient)
@@ -436,14 +444,15 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			var logs string
 			logsInBytes, _ := ioutil.ReadFile("../../test_files/mcs_logs.txt")
 			logs = string(logsInBytes)
+			infraEnvID := strfmt.UUID("eb82821f-bf21-4614-9a3b-ecb07929f250")
 			node0Id := strfmt.UUID("eb82821f-bf21-4614-9a3b-ecb07929f238")
 			node1Id := strfmt.UUID("eb82821f-bf21-4614-9a3b-ecb07929f239")
 			node2Id := strfmt.UUID("eb82821f-bf21-4614-9a3b-ecb07929f240")
 
-			testInventoryIdsIps := map[string]inventory_client.HostData{"node0": {Host: &models.Host{ID: &node0Id, Progress: &models.HostProgressInfo{CurrentStage: models.HostStageRebooting}},
+			testInventoryIdsIps := map[string]inventory_client.HostData{"node0": {Host: &models.Host{InfraEnvID: infraEnvID, ID: &node0Id, Progress: &models.HostProgressInfo{CurrentStage: models.HostStageRebooting}},
 				IPs: []string{"192.168.126.10", "192.168.11.122", "fe80::5054:ff:fe9a:4738"}},
-				"node1": {Host: &models.Host{ID: &node1Id, Progress: &models.HostProgressInfo{CurrentStage: models.HostStageRebooting}}, IPs: []string{"192.168.126.11", "192.168.11.123", "fe80::5054:ff:fe9a:4739"}},
-				"node2": {Host: &models.Host{ID: &node2Id, Progress: &models.HostProgressInfo{CurrentStage: models.HostStageRebooting}}, IPs: []string{"192.168.126.12", "192.168.11.124", "fe80::5054:ff:fe9a:4740"}}}
+				"node1": {Host: &models.Host{InfraEnvID: infraEnvID, ID: &node1Id, Progress: &models.HostProgressInfo{CurrentStage: models.HostStageRebooting}}, IPs: []string{"192.168.126.11", "192.168.11.123", "fe80::5054:ff:fe9a:4739"}},
+				"node2": {Host: &models.Host{InfraEnvID: infraEnvID, ID: &node2Id, Progress: &models.HostProgressInfo{CurrentStage: models.HostStageRebooting}}, IPs: []string{"192.168.126.12", "192.168.11.124", "fe80::5054:ff:fe9a:4740"}}}
 			mockbmclient.EXPECT().GetEnabledHostsNamesHosts(gomock.Any(), gomock.Any()).Return(nil, fmt.Errorf("dummy")).Times(1)
 			mockbmclient.EXPECT().GetEnabledHostsNamesHosts(gomock.Any(), gomock.Any()).Return(testInventoryIdsIps, nil).Times(1)
 			mockops.EXPECT().GetMCSLogs().Return("", fmt.Errorf("dummy")).Times(1)
@@ -451,9 +460,9 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			mockops.EXPECT().GetMCSLogs().Return("dummy logs", nil).Times(1)
 			mockops.EXPECT().GetMCSLogs().Return(logs, nil).AnyTimes()
 
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), gomock.Any(), models.HostStageConfiguring, gomock.Any()).Return(fmt.Errorf("dummy")).Times(1)
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), "eb82821f-bf21-4614-9a3b-ecb07929f240", models.HostStageConfiguring, gomock.Any()).Return(nil).Times(1)
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), "eb82821f-bf21-4614-9a3b-ecb07929f239", models.HostStageConfiguring, gomock.Any()).Return(nil).Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), gomock.Any(), gomock.Any(), models.HostStageConfiguring, gomock.Any()).Return(fmt.Errorf("dummy")).Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), "eb82821f-bf21-4614-9a3b-ecb07929f250", "eb82821f-bf21-4614-9a3b-ecb07929f240", models.HostStageConfiguring, gomock.Any()).Return(nil).Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), "eb82821f-bf21-4614-9a3b-ecb07929f250", "eb82821f-bf21-4614-9a3b-ecb07929f239", models.HostStageConfiguring, gomock.Any()).Return(nil).Times(1)
 
 			ctx, cancel := context.WithTimeout(context.Background(), 3*time.Second)
 			defer cancel()
@@ -464,12 +473,13 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			var logs string
 			logsInBytes, _ := ioutil.ReadFile("../../test_files/mcs_logs.txt")
 			logs = string(logsInBytes)
+			infraEnvId := strfmt.UUID("eb82821f-bf21-4614-9a3b-ecb07929f250")
 			node1Id := strfmt.UUID("eb82821f-bf21-4614-9a3b-ecb07929f239")
 			node2Id := strfmt.UUID("eb82821f-bf21-4614-9a3b-ecb07929f240")
 
 			testInventoryIdsIps := map[string]inventory_client.HostData{
-				"node1": {Host: &models.Host{ID: &node1Id, Progress: &models.HostProgressInfo{CurrentStage: models.HostStageRebooting}}, IPs: []string{"192.168.126.11", "192.168.11.123", "fe80::5054:ff:fe9a:4739"}},
-				"node2": {Host: &models.Host{ID: &node2Id, Progress: &models.HostProgressInfo{CurrentStage: models.HostStageRebooting}}, IPs: []string{"192.168.126.12", "192.168.11.124", "fe80::5054:ff:fe9a:4740"}}}
+				"node1": {Host: &models.Host{InfraEnvID: infraEnvId, ID: &node1Id, Progress: &models.HostProgressInfo{CurrentStage: models.HostStageRebooting}}, IPs: []string{"192.168.126.11", "192.168.11.123", "fe80::5054:ff:fe9a:4739"}},
+				"node2": {Host: &models.Host{InfraEnvID: infraEnvId, ID: &node2Id, Progress: &models.HostProgressInfo{CurrentStage: models.HostStageRebooting}}, IPs: []string{"192.168.126.12", "192.168.11.124", "fe80::5054:ff:fe9a:4740"}}}
 			mockbmclient.EXPECT().GetEnabledHostsNamesHosts(gomock.Any(), gomock.Any()).Return(nil, fmt.Errorf("dummy")).Times(1)
 			mockbmclient.EXPECT().GetEnabledHostsNamesHosts(gomock.Any(), gomock.Any()).Return(testInventoryIdsIps, nil).Times(1)
 			mockops.EXPECT().GetMCSLogs().Return("", fmt.Errorf("dummy")).Times(1)
@@ -477,9 +487,9 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			mockops.EXPECT().GetMCSLogs().Return("dummy logs", nil).Times(1)
 			mockops.EXPECT().GetMCSLogs().Return(logs, nil).AnyTimes()
 
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), gomock.Any(), models.HostStageConfiguring, gomock.Any()).Return(fmt.Errorf("dummy")).Times(1)
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), "eb82821f-bf21-4614-9a3b-ecb07929f240", models.HostStageConfiguring, gomock.Any()).Return(nil).Times(1)
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), "eb82821f-bf21-4614-9a3b-ecb07929f239", models.HostStageConfiguring, gomock.Any()).Return(nil).Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), gomock.Any(), gomock.Any(), models.HostStageConfiguring, gomock.Any()).Return(fmt.Errorf("dummy")).Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), "eb82821f-bf21-4614-9a3b-ecb07929f250", "eb82821f-bf21-4614-9a3b-ecb07929f240", models.HostStageConfiguring, gomock.Any()).Return(nil).Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), "eb82821f-bf21-4614-9a3b-ecb07929f250", "eb82821f-bf21-4614-9a3b-ecb07929f239", models.HostStageConfiguring, gomock.Any()).Return(nil).Times(1)
 			ctx, cancel := context.WithCancel(context.Background())
 			defer cancel()
 			installerObj.updateConfiguringStatus(ctx)
@@ -489,6 +499,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 		installerArgs := []string{"-n", "--append-karg", "nameserver=8.8.8.8"}
 		conf := config.Config{Role: string(models.HostRoleMaster),
 			ClusterID:        "cluster-id",
+			InfraEnvID:       "infra-env-id",
 			HostID:           "host-id",
 			Device:           "/dev/vda",
 			URL:              "https://assisted-service.com:80",
@@ -508,7 +519,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			})
 			cleanInstallDevice()
 			mkdirSuccess(InstallDir)
-			downloadHostIgnitionSuccess(hostId, "master-host-id.ign")
+			downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
 			writeToDiskSuccess(installerArgs)
 			setBootOrderSuccess(gomock.Any())
 			uploadLogsSuccess(false)
@@ -521,6 +532,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			cleanInstallDeviceClean := func() {
 				mockops.EXPECT().GetVGByPV(device).Return("vg1", nil).Times(1)
 				mockops.EXPECT().RemoveVG("vg1").Return(nil).Times(1)
+				mockops.EXPECT().IsRaidDevice(device).Return(false).Times(1)
 				mockops.EXPECT().Wipefs(device).Return(nil).Times(1)
 				mockops.EXPECT().RemovePV(device).Return(nil).Times(1)
 			}
@@ -542,6 +554,33 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			ret := installerObj.InstallNode()
 			Expect(ret).Should(Equal(err))
 		})
+		It("HostRoleMaster role raid cleanup disk - happy flow", func() {
+			cleanInstallDeviceClean := func() {
+				mockops.EXPECT().GetVGByPV(device).Return("", nil).Times(1)
+				mockops.EXPECT().IsRaidDevice(device).Return(true).Times(1)
+				mockops.EXPECT().CleanRaidDevice(device).Return(nil).Times(1)
+				mockops.EXPECT().Wipefs(device).Return(nil).Times(1)
+			}
+			updateProgressSuccess([][]string{{string(models.HostStageStartingInstallation), conf.Role}})
+			cleanInstallDeviceClean()
+			err := fmt.Errorf("failed to create dir")
+			mockops.EXPECT().Mkdir(InstallDir).Return(err).Times(1)
+			ret := installerObj.InstallNode()
+			Expect(ret).Should(Equal(err))
+		})
+		It("HostRoleMaster role raid cleanup disk - failed", func() {
+			err := fmt.Errorf("failed cleaning raid device")
+
+			cleanInstallDeviceClean := func() {
+				mockops.EXPECT().GetVGByPV(device).Return("", nil).Times(1)
+				mockops.EXPECT().IsRaidDevice(device).Return(true).Times(1)
+				mockops.EXPECT().CleanRaidDevice(device).Return(err).Times(1)
+			}
+			updateProgressSuccess([][]string{{string(models.HostStageStartingInstallation), conf.Role}})
+			cleanInstallDeviceClean()
+			ret := installerObj.InstallNode()
+			Expect(ret).Should(Equal(err))
+		})
 		It("HostRoleMaster role failed to create dir", func() {
 			updateProgressSuccess([][]string{{string(models.HostStageStartingInstallation), conf.Role}})
 			cleanInstallDevice()
@@ -557,7 +596,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			cleanInstallDevice()
 			mkdirSuccess(InstallDir)
 			err := fmt.Errorf("failed to fetch file")
-			mockbmclient.EXPECT().DownloadHostIgnition(gomock.Any(), hostId, filepath.Join(InstallDir, "master-host-id.ign")).Return(err).Times(1)
+			mockbmclient.EXPECT().DownloadHostIgnition(gomock.Any(), infraEnvId, hostId, filepath.Join(InstallDir, "master-host-id.ign")).Return(err).Times(1)
 			ret := installerObj.InstallNode()
 			Expect(ret).Should(Equal(err))
 		})
@@ -568,7 +607,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			})
 			cleanInstallDevice()
 			mkdirSuccess(InstallDir)
-			downloadHostIgnitionSuccess(hostId, "master-host-id.ign")
+			downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
 			err := fmt.Errorf("failed to write image to disk")
 			mockops.EXPECT().WriteImageToDisk(filepath.Join(InstallDir, "master-host-id.ign"), device, mockbmclient, installerArgs).Return(err).Times(3)
 			ret := installerObj.InstallNode()
@@ -582,7 +621,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			})
 			cleanInstallDevice()
 			mkdirSuccess(InstallDir)
-			downloadHostIgnitionSuccess(hostId, "master-host-id.ign")
+			downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
 			uploadLogsSuccess(false)
 			reportLogProgressSuccess()
 			writeToDiskSuccess(installerArgs)
@@ -596,6 +635,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 	Context("Worker role", func() {
 		conf := config.Config{Role: string(models.HostRoleWorker),
 			ClusterID:        "cluster-id",
+			InfraEnvID:       "infra-env-id",
 			HostID:           "host-id",
 			Device:           "/dev/vda",
 			URL:              "https://assisted-service.com:80",
@@ -609,11 +649,29 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			updateProgressSuccess([][]string{{string(models.HostStageStartingInstallation), conf.Role},
 				{string(models.HostStageInstalling), conf.Role},
 				{string(models.HostStageWritingImageToDisk)},
+				{string(models.HostStageWaitingForControlPlane)},
 				{string(models.HostStageRebooting)},
 			})
+			cluster := models.Cluster{}
+			hosts := models.HostList{
+				{
+					Role: models.HostRoleMaster,
+					Progress: &models.HostProgressInfo{
+						CurrentStage: models.HostStageDone,
+					},
+				},
+				{
+					Role: models.HostRoleMaster,
+					Progress: &models.HostProgressInfo{
+						CurrentStage: models.HostStageDone,
+					},
+				},
+			}
+			mockbmclient.EXPECT().GetCluster(gomock.Any(), false).Return(&cluster, nil).Times(1)
+			mockbmclient.EXPECT().ListsHostsForRole(gomock.Any(), "master").Return(hosts, nil).Times(1)
 			cleanInstallDevice()
 			mkdirSuccess(InstallDir)
-			downloadHostIgnitionSuccess(hostId, "worker-host-id.ign")
+			downloadHostIgnitionSuccess(infraEnvId, hostId, "worker-host-id.ign")
 			mockops.EXPECT().WriteImageToDisk(filepath.Join(InstallDir, "worker-host-id.ign"), device, mockbmclient, nil).Return(nil).Times(1)
 			setBootOrderSuccess(gomock.Any())
 			// failure must do nothing
@@ -628,6 +686,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 
 		conf := config.Config{Role: string(models.HostRoleMaster),
 			ClusterID:            "cluster-id",
+			InfraEnvID:           "infra-env-id",
 			HostID:               "host-id",
 			Device:               "/dev/vda",
 			URL:                  "https://assisted-service.com:80",
@@ -674,7 +733,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			mockops.EXPECT().PrepareController().Return(nil).Times(1)
 		}
 		waitForBootkubeSuccess := func() {
-			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), hostId, models.HostStageWaitingForBootkube, "").Return(nil).Times(1)
+			mockbmclient.EXPECT().UpdateHostInstallProgress(gomock.Any(), infraEnvId, hostId, models.HostStageWaitingForBootkube, "").Return(nil).Times(1)
 			mockops.EXPECT().ExecPrivilegeCommand(gomock.Any(), "stat", "/opt/openshift/.bootkube.done").Return("OK", nil).Times(1)
 		}
 		bootkubeStatusSuccess := func() {
@@ -686,6 +745,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 		singleNodeBootstrapSetup := func() {
 			cleanInstallDevice()
 			mkdirSuccess(InstallDir)
+			mkdirSuccess(sshDir)
 			downloadFileSuccess(bootstrapIgn)
 			extractSecretFromIgnitionSuccess()
 			extractIgnitionToFS("Success", nil)
@@ -712,7 +772,7 @@ var _ = Describe("installer HostRoleMaster role", func() {
 			//HostRoleMaster flow:
 			verifySingleNodeMasterIgnitionSuccess()
 			singleNodeMergeIgnitionSuccess()
-			downloadHostIgnitionSuccess(hostId, "master-host-id.ign")
+			downloadHostIgnitionSuccess(infraEnvId, hostId, "master-host-id.ign")
 			mockops.EXPECT().WriteImageToDisk(singleNodeMasterIgnitionPath, device, mockbmclient, nil).Return(nil).Times(1)
 			setBootOrderSuccess(gomock.Any())
 			uploadLogsSuccess(true)
