@@ -5,6 +5,8 @@ import (
 	"fmt"
 	"io"
 	"os"
+	"regexp"
+	"runtime"
 	"strconv"
 	"text/template"
 
@@ -23,6 +25,11 @@ import (
 	"github.com/openshift/assisted-installer/src/utils"
 )
 
+const (
+	coreosInstallerExecutable       = "coreos-installer"
+	dryRunCoreosInstallerExecutable = "dry-installer"
+)
+
 //go:generate mockgen -source=ops.go -package=ops -destination=mock_ops.go
 type Ops interface {
 	ExecPrivilegeCommand(liveLogger io.Writer, command string, args ...string) (string, error)
@@ -39,6 +46,8 @@ type Ops interface {
 	RemoveLV(lvName, vgName string) error
 	RemovePV(pvName string) error
 	Wipefs(device string) error
+	IsRaidDevice(device string) bool
+	CleanRaidDevice(device string) error
 	GetMCSLogs() (string, error)
 	UploadInstallationLogs(isBootstrap bool) (string, error)
 	ReloadHostFile(filepath string) error
@@ -47,7 +56,9 @@ type Ops interface {
 	CreateRandomHostname(hostname string) error
 	GetHostname() (string, error)
 	EvaluateDiskSymlink(string) string
-	CreateManifests(string, string) error
+	FormatDisk(string) error
+	CreateManifests(string, []byte) error
+	DryRebootHappened(markerPath string) bool
 }
 
 const (
@@ -85,9 +96,30 @@ func NewOps(logger *logrus.Logger, proxySet bool) Ops {
 }
 
 // ExecPrivilegeCommand execute a command in the host environment via nsenter
+
 func (o *ops) ExecPrivilegeCommand(liveLogger io.Writer, command string, args ...string) (string, error) {
+	// nsenter is used here to launch processes inside the container in a way that makes said processes feel
+	// and behave as if they're running on the host directly rather than inside the container
 	commandBase := "nsenter"
-	arguments := []string{"-t", "1", "-m", "-i", "--", command}
+
+	arguments := []string{
+		"--target", "1",
+		// Entering the cgroup namespace is not required for podman on CoreOS (where the
+		// agent typically runs), but it's needed on some Fedora versions and
+		// some other systemd based systems. Those systems are used to run dry-mode
+		// agents for load testing. If this flag is not used, Podman will sometimes
+		// have trouble creating a systemd cgroup slice for new containers.
+		"--cgroup",
+		// The mount namespace is required for podman to access the host's container
+		// storage
+		"--mount",
+		// TODO: Document why we need the IPC namespace
+		"--ipc",
+		"--pid",
+		"--",
+		command,
+	}
+
 	arguments = append(arguments, args...)
 	return o.ExecCommand(liveLogger, commandBase, arguments...)
 }
@@ -101,17 +133,24 @@ type ExecCommandError struct {
 	WaitStatus int
 }
 
+func removePullSecret(s []string) []string {
+	if config.GlobalConfig.PullSecretToken == "" {
+		return s
+	}
+
+	return strings.Split(strings.ReplaceAll(strings.Join(s, " "), config.GlobalConfig.PullSecretToken, "<SECRET>"), " ")
+}
+
 func (e *ExecCommandError) Error() string {
 	lastOutput := e.Output
 	if len(e.Output) > 200 {
 		lastOutput = "... " + e.Output[len(e.Output)-200:]
 	}
-
-	return fmt.Sprintf("failed executing %s %v, Error %s, LastOutput \"%s\"", e.Command, e.Args, e.ExitErr, lastOutput)
+	return fmt.Sprintf("failed executing %s %v, Error %s, LastOutput \"%s\"", e.Command, removePullSecret(e.Args), e.ExitErr, lastOutput)
 }
 
 func (e *ExecCommandError) DetailedError() string {
-	return fmt.Sprintf("failed executing %s %v, env vars %v, error %s, waitStatus %d, Output \"%s\"", e.Command, e.Args, e.Env, e.ExitErr, e.WaitStatus, e.Output)
+	return fmt.Sprintf("failed executing %s %v, env vars %v, error %s, waitStatus %d, Output \"%s\"", e.Command, removePullSecret(e.Args), removePullSecret(e.Env), e.ExitErr, e.WaitStatus, e.Output)
 }
 
 // ExecCommand executes command.
@@ -156,7 +195,7 @@ func (o *ops) ExecCommand(liveLogger io.Writer, command string, args ...string)
 		}
 		return output, execErr
 	}
-	o.log.Debug("Command executed:", " command", command, " arguments", args, "env vars", cmd.Env, "output", output)
+	o.log.Debug("Command executed:", " command", command, " arguments", removePullSecret(args), "env vars", removePullSecret(cmd.Env), "output", output)
 	return output, err
 }
 
@@ -167,6 +206,10 @@ func (o *ops) Mkdir(dirName string) error {
 }
 
 func (o *ops) SystemctlAction(action string, args ...string) error {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
+	}
+
 	o.log.Infof("Running systemctl %s %s", action, args)
 	_, err := o.ExecPrivilegeCommand(o.logWriter, "systemctl", append([]string{action}, args...)...)
 	if err != nil {
@@ -178,8 +221,17 @@ func (o *ops) SystemctlAction(action string, args ...string) error {
 func (o *ops) WriteImageToDisk(ignitionPath string, device string, progressReporter inventory_client.InventoryClient, extraArgs []string) error {
 	allArgs := installerArgs(ignitionPath, device, extraArgs)
 	o.log.Infof("Writing image and ignition to disk with arguments: %v", allArgs)
-	_, err := o.ExecPrivilegeCommand(NewCoreosInstallerLogWriter(o.log, progressReporter, config.GlobalConfig.HostID),
-		"coreos-installer", allArgs...)
+
+	installerExecutable := coreosInstallerExecutable
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		// In dry run, we use an executable called dry-installer rather than coreos-installer.
+		// This executable is expected to pretend to be doing coreos-installer stuff and print fake
+		// progress. It's up to the dry-mode user to make sure such executable is available in PATH
+		installerExecutable = dryRunCoreosInstallerExecutable
+	}
+
+	_, err := o.ExecPrivilegeCommand(NewCoreosInstallerLogWriter(o.log, progressReporter, config.GlobalConfig.InfraEnvID, config.GlobalConfig.HostID),
+		installerExecutable, allArgs...)
 	return err
 }
 
@@ -201,6 +253,20 @@ func (o *ops) EvaluateDiskSymlink(device string) string {
 	return device
 }
 
+func (o *ops) FormatDisk(disk string) error {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
+	}
+
+	o.log.Infof("Formatting disk %s", disk)
+	_, err := o.ExecPrivilegeCommand(o.logWriter, "dd", "if=/dev/zero", fmt.Sprintf("of=%s", disk), "bs=512", "count=1")
+	if err != nil {
+		o.log.Errorf("Failed to format disk %s, err: %s", disk, err)
+		return err
+	}
+	return nil
+}
+
 func installerArgs(ignitionPath string, device string, extra []string) []string {
 	allArgs := []string{"install", "--insecure", "-i", ignitionPath}
 	if extra != nil {
@@ -210,6 +276,11 @@ func installerArgs(ignitionPath string, device string, extra []string) []string
 }
 
 func (o *ops) Reboot() error {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		_, err := o.ExecPrivilegeCommand(o.logWriter, "touch", config.GlobalDryRunConfig.FakeRebootMarkerPath)
+		return errors.Wrap(err, "failed to touch fake reboot marker")
+	}
+
 	o.log.Info("Rebooting node")
 	_, err := o.ExecPrivilegeCommand(o.logWriter, "shutdown", "-r", "+1", "'Installation completed, server is going to reboot.'")
 	if err != nil {
@@ -220,6 +291,10 @@ func (o *ops) Reboot() error {
 }
 
 func (o *ops) SetBootOrder(device string) error {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
+	}
+
 	_, err := o.ExecPrivilegeCommand(nil, "test", "-d", "/sys/firmware/efi")
 	if err != nil {
 		o.log.Info("setting the boot order on BIOS systems is not supported. Skipping...")
@@ -229,7 +304,7 @@ func (o *ops) SetBootOrder(device string) error {
 	o.log.Info("Setting efibootmgr to boot from disk")
 
 	// efi-system is installed onto partition 2
-	_, err = o.ExecPrivilegeCommand(o.logWriter, "efibootmgr", "-d", device, "-p", "2", "-c", "-L", "Red Hat Enterprise Linux", "-l", "\\EFI\\redhat\\shimx64.efi")
+	_, err = o.ExecPrivilegeCommand(o.logWriter, "efibootmgr", "-v", "-d", device, "-p", "2", "-c", "-L", "Red Hat Enterprise Linux", "-l", o.getEfiFilePath())
 	if err != nil {
 		o.log.Errorf("Failed to set efibootmgr to boot from disk %s, err: %s", device, err)
 		return err
@@ -237,8 +312,24 @@ func (o *ops) SetBootOrder(device string) error {
 	return nil
 }
 
+func (o *ops) getEfiFilePath() string {
+	var efiFileName string
+	switch runtime.GOARCH {
+	case "arm64":
+		efiFileName = "shimaa64.efi"
+	default:
+		efiFileName = "shimx64.efi"
+	}
+	o.log.Infof("Using EFI file '%s' for GOARCH '%s'", efiFileName, runtime.GOARCH)
+	return fmt.Sprintf("\\EFI\\redhat\\%s", efiFileName)
+}
+
 func (o *ops) ExtractFromIgnition(ignitionPath string, fileToExtract string) error {
-	o.log.Infof("Getting pull secret from %s", ignitionPath)
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
+	}
+
+	o.log.Infof("Getting data from %s", ignitionPath)
 	ignitionData, err := ioutil.ReadFile(ignitionPath)
 	if err != nil {
 		o.log.Errorf("Error occurred while trying to read %s : %e", ignitionPath, err)
@@ -418,14 +509,147 @@ func (o *ops) RemovePV(pvName string) error {
 }
 
 func (o *ops) Wipefs(device string) error {
-	output, err := o.ExecPrivilegeCommand(o.logWriter, "wipefs", "-a", device)
+	_, err := o.ExecPrivilegeCommand(o.logWriter, "wipefs", "--all", "--force", device)
+
 	if err != nil {
-		o.log.Errorf("Failed to wipefs device %s, output %s, error %s", device, output, err)
+		_, err = o.ExecPrivilegeCommand(o.logWriter, "wipefs", "--all", device)
 	}
+
 	return err
 }
 
+func (o *ops) IsRaidDevice(device string) bool {
+	raidDevices, err := o.getRaidDevices()
+
+	if err != nil {
+		o.log.WithError(err).Errorf("Error occurred while trying to get list of raid devices - continue without cleaning")
+		return false
+	}
+
+	// The device itself or one of its partitions
+	expression, _ := regexp.Compile(device + "[\\d]*")
+
+	for _, raidArrayMembers := range raidDevices {
+		for _, raidMember := range raidArrayMembers {
+			if expression.MatchString(raidMember) {
+				return true
+			}
+		}
+	}
+
+	return false
+}
+
+func (o *ops) CleanRaidDevice(device string) error {
+	raidDevices, err := o.getRaidDevices()
+
+	if err != nil {
+		return err
+	}
+
+	for raidDeviceName, raidArrayMembers := range raidDevices {
+		err = o.cleanDeviceFromRaidDevice(device, raidDeviceName, raidArrayMembers)
+
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+func (o *ops) getRaidDevices() (map[string][]string, error) {
+	output, err := o.ExecPrivilegeCommand(o.logWriter, "mdadm", "-v", "--query", "--detail", "--scan")
+
+	if err != nil {
+		return nil, err
+	}
+
+	lines := strings.Split(output, "\n")
+	result := make(map[string][]string)
+
+	/*
+		The output pattern is:
+		ARRAY /dev/md0 level=raid1 num-devices=2 metadata=1.2 name=0 UUID=77e1b6f2:56530ebd:38bd6808:17fd01c4
+		   devices=/dev/vda2,/dev/vda3
+		ARRAY /dev/md1 level=raid1 num-devices=1 metadata=1.2 name=1 UUID=aad7aca9:81db82f3:2f1fedb1:f89ddb43
+		   devices=/dev/vda1
+	*/
+	for i := 0; i < len(lines); {
+		if !strings.Contains(lines[i], "ARRAY") {
+			i++
+			continue
+		}
+
+		fields := strings.Fields(lines[i])
+		raidDeviceName := fields[1]
+		i++
+
+		// Ensuring that we have at least two lines per device.
+		if len(lines) == i {
+			break
+		}
+
+		raidArrayMembersStr := strings.TrimSpace(lines[i])
+		prefix := "devices="
+
+		if !strings.HasPrefix(raidArrayMembersStr, prefix) {
+			continue
+		}
+
+		raidArrayMembersStr = raidArrayMembersStr[len(prefix):]
+		result[raidDeviceName] = strings.Split(raidArrayMembersStr, ",")
+		i++
+	}
+
+	return result, nil
+}
+
+func (o *ops) cleanDeviceFromRaidDevice(deviceName string, raidDeviceName string, raidArrayMembers []string) error {
+	raidStopped := false
+
+	expression, _ := regexp.Compile(deviceName + "[\\d]*")
+
+	for _, raidMember := range raidArrayMembers {
+		// A partition or the device itself is part of the raid array.
+		if expression.MatchString(raidMember) {
+			// Stop the raid device.
+			if !raidStopped {
+				o.log.Info("Stopping raid device: " + raidDeviceName)
+				_, err := o.ExecPrivilegeCommand(o.logWriter, "mdadm", "--stop", raidDeviceName)
+
+				if err != nil {
+					return err
+				}
+
+				raidStopped = true
+			}
+
+			// Clean the raid superblock from the device
+			o.log.Infof("Cleaning raid member %s superblock", raidMember)
+			_, err := o.ExecPrivilegeCommand(o.logWriter, "mdadm", "--zero-superblock", raidMember)
+
+			if err != nil {
+				return err
+			}
+		}
+	}
+	return nil
+}
+
 func (o *ops) GetMCSLogs() (string, error) {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		mcsLogs := ""
+		for _, clusterHost := range config.GlobalDryRunConfig.ParsedClusterHosts {
+			// Add IP access log for each IP, this is how the installer determines which node has downloaded the ignition
+			if !o.DryRebootHappened(clusterHost.RebootMarkerPath) {
+				// Host didn't even reboot yet, don't pretend it fetched the ignition
+				continue
+			}
+			mcsLogs += fmt.Sprintf("%s.(Ignition)\n", clusterHost.Ip)
+		}
+		return mcsLogs, nil
+	}
 
 	files, err := utils.FindFiles("/var/log/containers/", utils.W_FILEONLY, "*machine-config-server*.log")
 	if err != nil {
@@ -447,12 +671,16 @@ func (o *ops) GetMCSLogs() (string, error) {
 	return string(logs), nil
 }
 
+// This function actually runs container that imeplements logs_sender command
+// Any change to the assisted-service API that is used by the logs_sender command
+// ( for example UploadLogs), must be reflected here (input parameters, etc'),
+// if needed
 func (o *ops) UploadInstallationLogs(isBootstrap bool) (string, error) {
 	command := "podman"
 	args := []string{"run", "--rm", "--privileged", "--net=host", "--pid=host", "-v", "/run/systemd/journal/socket:/run/systemd/journal/socket",
 		"-v", "/var/log:/var/log", config.GlobalConfig.AgentImage, "logs_sender",
 		"-cluster-id", config.GlobalConfig.ClusterID, "-url", config.GlobalConfig.URL,
-		"-host-id", config.GlobalConfig.HostID,
+		"-host-id", config.GlobalConfig.HostID, "-infra-env-id", config.GlobalConfig.InfraEnvID,
 		"-pull-secret-token", config.GlobalConfig.PullSecretToken,
 		fmt.Sprintf("-insecure=%s", strconv.FormatBool(config.GlobalConfig.SkipCertVerification)),
 		fmt.Sprintf("-bootstrap=%s", strconv.FormatBool(isBootstrap)),
@@ -468,6 +696,10 @@ func (o *ops) UploadInstallationLogs(isBootstrap bool) (string, error) {
 // For example /etc/resolv.conf, it can't be changed with Z flag but is updated by bootkube.sh
 // and we need this update for dns resolve of kubeapi
 func (o *ops) ReloadHostFile(filepath string) error {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
+	}
+
 	o.log.Infof("Reloading %s", filepath)
 	output, err := o.ExecPrivilegeCommand(o.logWriter, "cat", filepath)
 	if err != nil {
@@ -491,6 +723,10 @@ func (o *ops) ReloadHostFile(filepath string) error {
 }
 
 func (o *ops) CreateOpenshiftSshManifest(filePath, tmpl, sshPubKeyPath string) error {
+	if config.GlobalDryRunConfig.DryRunEnabled {
+		return nil
+	}
+
 	o.log.Info("Create an openshift manifets for SSH public key")
 	sshPublicKey, err := o.ExecPrivilegeCommand(o.logWriter, "cat", sshPubKeyPath)
 	if err != nil {
@@ -565,13 +801,35 @@ func (o *ops) GetHostname() (string, error) {
 	return os.Hostname()
 }
 
-func (o *ops) CreateManifests(kubeconfig string, manifestFilePath string) error {
-	command := fmt.Sprintf("oc --kubeconfig=%s apply -f %s", kubeconfig, manifestFilePath)
+func (o *ops) CreateManifests(kubeconfig string, content []byte) error {
+	// Create temp file, where we store the content to be create by oc command:
+	file, err := ioutil.TempFile("", "operator-manifest")
+	if err != nil {
+		return err
+	}
+	defer os.Remove(file.Name())
+
+	// Write the content to the temporary file:
+	if err = ioutil.WriteFile(file.Name(), content, 0644); err != nil {
+		return err
+	}
+
+	// Run oc command that creates the custom manifest:
+	command := fmt.Sprintf("oc --kubeconfig=%s apply -f %s", kubeconfig, file.Name())
 	output, err := o.ExecCommand(o.logWriter, "bash", "-c", command)
 	if err != nil {
 		return err
 	}
-	o.log.Infof("Applying custom manifest file %s succeed %s", manifestFilePath, output)
+	o.log.Infof("Applying custom manifest file %s succeed %s", file.Name(), output)
 
 	return nil
 }
+
+// DryRebootHappened checks if a reboot happened according to a particular reboot marker path
+// The dry run installer creates this file on "Reboot" (instead of actually rebooting)
+// We use this function to check whether the given node in the cluster have already rebooted
+func (o *ops) DryRebootHappened(markerPath string) bool {
+	_, err := o.ExecPrivilegeCommand(nil, "stat", markerPath)
+
+	return err == nil
+}
